/***********************************************************************
 * Dateiname         :  FrSkyProtokoll.ino                             *
 *                                                                     *
 * Projekt,Version   :  Version 0.3                                    *
 *                                                                     *
 * Compiler-Version  : Arduino 1.6.7                                   *
 *                                                                     *
 * Prozessor         : Teensy 3.2                                      *
 *                                                                     *
 * Unterprogramme    :                                                 *
 *                                                                     *
 * Kurzbeschreibung  : Mit diesem Programmteil wird die Umsetzung auf  *
 *                     das FrSky HUB-Protokoll bewerkstelligt.         *
 *                                                                     *
 * zus. Dokument.    : keine                                           *
 *                                                                     *
 * Hinweise          :                                                 *
 *                                                                     *
 *                                                                     *
 * Original erstellt von T.Pfaff für ACT                               *
 *                                                                     *
 * geaendert         :                                                 *
 *                                                                     * 
 *                                                                     *
 ***********************************************************************/

// ************************************************************
// *** Output for remote voltage A1 and A2 and link quality ***
// *** Header 0x7E mit Prim 0xFE                            ***
// ************************************************************

static void sendRSSI(uint8_t ad0,uint8_t ad1,uint8_t rssi)
{
    Serial1.write(PROTOCOL_HEADER);//Start
    Serial1.write(PROTOCOL_RSSI);//User

    //byte stuffing
    if (ad0 == PROTOCOL_HEADER) 
    {
        Serial1.write(PROTOCOL_STUF);
        Serial1.write(PROTOCOL_STUFX0);
    } 
    else if (ad0 == PROTOCOL_STUF) 
    {
        Serial1.write(PROTOCOL_STUF);
        Serial1.write(PROTOCOL_STUFX1);
    } 
    else
    {
      Serial1.write(ad0);
    }  

    //byte stuffing
    if (ad1 == PROTOCOL_HEADER) 
    {
        Serial1.write(PROTOCOL_STUF);
        Serial1.write(PROTOCOL_STUFX0);
    } 
    else if (ad1 == PROTOCOL_STUF) 
    {
        Serial1.write(PROTOCOL_STUF);
        Serial1.write(PROTOCOL_STUFX1);
    } 
    else
    {
      Serial1.write(ad1);
    } 


    //byte stuffing
    if (rssi == PROTOCOL_HEADER) 
    {
        Serial1.write(PROTOCOL_STUF);
        Serial1.write(PROTOCOL_STUFX0);
    } 
    else if (rssi == PROTOCOL_STUF) 
    {
        Serial1.write(PROTOCOL_STUF);
        Serial1.write(PROTOCOL_STUFX1);
    } 
    else
    {
      Serial1.write(rssi);
    } 
    
    Serial1.write((byte)0);
    Serial1.write((byte)0);
    Serial1.write((byte)0);
    Serial1.write((byte)0);
    Serial1.write((byte)0);
    Serial1.write(PROTOCOL_TAIL);
}

// ************************************************************
// *** For User-Data use Header 0x7E with Prim 0xFD         ***
// ************************************************************
static void serializeFrsky(uint8_t id0, int16_t a0)
{

    uint8_t data[2], hub[15]={PROTOCOL_HUB_HEADER,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
    frame[40]={PROTOCOL_HEADER,PROTOCOL_PRIM,5,0,0,0,0,0,0,0,
                             0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                             0,0,0,0,0,0,0,0,0,0},
    hub_index=1,frame_index=4;
    data[0] = a0&0x00FF;
    data[1] = (a0 >> 8) & 0xff;  

    //generating hub protocol with byte stuffing
    if(id0 == PROTOCOL_HUB_HEADER) 
    { 
      hub[hub_index++]=PROTOCOL_HUB_STUF; 
      hub[hub_index++]=PROTOCOL_HUB_STUFX0;   
    } 
    else if (id0 == PROTOCOL_HUB_STUF)   
    { 
      hub[hub_index++]=PROTOCOL_HUB_STUF; 
      hub[hub_index++]=PROTOCOL_HUB_STUFX1;   
    }     
    else hub[hub_index++]=id0;
      
    for(byte index=0; index < 2;index++)
    {
      if      (data[index] == PROTOCOL_HUB_HEADER) 
      { 
        hub[hub_index++]=PROTOCOL_HUB_STUF; 
        hub[hub_index++]=PROTOCOL_HUB_STUFX0; 
      } 
      else if (data[index] == PROTOCOL_HUB_STUF)   
      { 
        hub[hub_index++]=PROTOCOL_HUB_STUF; 
        hub[hub_index++]=PROTOCOL_HUB_STUFX1; 
      }     
      else hub[hub_index++]=data[index];
    }
    hub[hub_index]=PROTOCOL_HUB_TAIL;
    for(byte index=hub_index ; index<16 ;index++)
    {
      hub[index]=0x00;
    }

    //generate frame protocol
    frame[0]=PROTOCOL_HEADER;
    frame[1]=PROTOCOL_PRIM;
    if(hub_index==4)
      frame[2]=5;
    else if (hub_index>=5)   
      frame[2]=6;
    frame[3]=0x00;
      frame[4]=hub[0];    
      frame[5]=hub[1];    
      frame[6]=hub[2];    
      frame[7]=hub[3];    
      frame[8]=hub[4];       
      frame[9]=hub[5];
    frame[10]=PROTOCOL_TAIL;
    frame_index=10;

    //if more than 6 bytes to send secound frame
    if(hub_index > 5)     
    {
      frame[11+0]=PROTOCOL_HEADER;
      frame[11+1]=PROTOCOL_PRIM;
      if(hub_index<11)
        frame[11+2]=hub_index-5;
      else  
        frame[11+2]=6;
      frame[11+3]=0x00;
        frame[11+4]=hub[6+0];    
        frame[11+5]=hub[6+1];    
        frame[11+6]=hub[6+2];    
        frame[11+7]=hub[6+3];    
        frame[11+8]=hub[6+4];       
        frame[11+9]=hub[6+5];
      frame[11+10]=PROTOCOL_TAIL;
      frame_index=21;
    }  
    //byte stuffing and sending
    for(byte index=0; index<= frame_index;index++)
    {
     if (frame[index] == PROTOCOL_HEADER && index!=0  && index!=11&& index!=10  && index!=21) 
      {
        Serial1.write(PROTOCOL_STUF);
        Serial1.write(PROTOCOL_STUFX0);
      } 
      else if (frame[index] == PROTOCOL_STUF  && index!=0  && index!=11  && index!=10  && index!=21 ) 
      {
         Serial1.write(PROTOCOL_STUF);
         Serial1.write(PROTOCOL_STUFX1);
      } 
      else
      {
        Serial1.write(frame[index]);
      }
    }
}

void sendZelle(byte ID, int16_t zelle)
{        
        serializeFrsky(ID,zelle);                    
}        
//---------------------------------------------------------        

//---------------------------------------------------------        
void sendTemp1(int16_t Tempf)
{        
        serializeFrsky(ID_TEMPRATURE1,Tempf);            
}        
//---------------------------------------------------------        
void sendTemp2(int16_t Tempf)
{        
        serializeFrsky(ID_TEMPRATURE2,Tempf);            
}        
//---------------------------------------------------------        
void sendCurr(uint16_t Stromf)
{        
        serializeFrsky(ID_CURRENT,Stromf);
}        
//---------------------------------------------------------        
void sendVolt(int16_t Uf)
{        
        serializeFrsky(ID_VOLTAGE_NEW,Uf);          
}//---------------------------------------------------------        
void sendVoltHD(uint16_t Uf)
{        
//        Uf *= 0.54;
        serializeFrsky(ID_VOLTHD_BP,Uf / 100);          
        serializeFrsky(ID_VOLTHD_AP,abs(Uf) % 100);          
}//---------------------------------------------------------        
void sendVoltZelle(int16_t Uf,byte zelle)
{        
        serializeFrsky(ID_VOLT,((((uint16_t)zelle&0x000F)<<12)&0xF000) | ((Uf>>1)&0x0FFF));                    
}        
//---------------------------------------------------------        
void sendAlt(int16_t Hochf)
{        
        
        serializeFrsky(ID_ALTITUDE_BP,(Hochf) / 100);//bei hoch werden negartive Werte aly byte überlauf angezeigt Bug in OpenTX
        //serializeFrsky(ID_ALTITUDE_BP,Hochf / 100);
        
        serializeFrsky(ID_ALTITUDE_AP,abs(Hochf) % 100);//Hier muss immer abs() verwendet werden           
}        
//---------------------------------------------------------        
void sendSpeed(int16_t v_horf)
{        
        serializeFrsky(ID_GPS_SPEED_BP,v_horf / 100);
        serializeFrsky(ID_GPS_SPEED_AP,v_horf % 100);           
}        
//---------------------------------------------------------        
void sendRpm(uint16_t rpmf)
{        
        serializeFrsky(ID_RPM,rpmf);                
}        
//---------------------------------------------------------        
void sendKap(uint16_t Kapf)
{        
        serializeFrsky(ID_FUEL_LEVEL,Kapf);        
}        
//---------------------------------------------------------        
void sendVario(int16_t v_steigf)
{        
        serializeFrsky(ID_V_VARIO,v_steigf);    
}        
//---------------------------------------------------------        
void sendKurs(uint16_t Grad_Kursf)
{        
        serializeFrsky(ID_COURSE_BP,Grad_Kursf / 100);
        serializeFrsky(ID_COURSE_AP,Grad_Kursf % 100);     
}
//---------------------------------------------------------        
void sendRSSI_RAW(uint16_t Rssif)
{        
        serializeFrsky(ID_RSSI_RAW,Rssif);
}

